{ Copyright (C) by Artem A. Berman   }
{ And modified by Sotiris Papadopoulos} 

{$D-,L-,Y-,I-,S-}

unit ralarm;

interface

uses Classes, Controls, Forms, SysUtils, StdCtrls, ExtCtrls;

type
  EAlarmException = class(Exception);

  TAlarmFromTo = class(TComponent)
  private
    FHour,
    FMin,
    FSec,
    FMlSec: Word;

    FAlarmTimeBegin: string;
    FAlarmTimeEnd: string;
    FShowObjectBegin: TControl;
    FShowObjectEnd: TControl;
    FTimer: TTimer;
    FOnAlarmBegin: TNotifyEvent;
    FOnAlarmEnd: TNotifyEvent;
    FActiveBegin: Boolean;
    FActiveEnd: Boolean;
    FInterval: Cardinal;
    FMSec: Boolean;
    FFormat: string;

    procedure SetActiveBegin(State: Boolean);
    procedure SetActiveEnd(State: Boolean);
    procedure SetAlarmTime(Tm: string);
    procedure SetAlarmTimeEnd(Tm: string);
    procedure SetInterval(Inter: Cardinal);
    procedure SetMSec(State: Boolean);
    procedure RunAlarm(Sender: TObject);
    procedure AlarmBegin;
    procedure AlarmEnd;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property AlarmTimeBegin: string read FAlarmTimeBegin write SetAlarmTime;
    property Interval: Cardinal read FInterval write SetInterval;
    property AlarmTimeEnd: string read FAlarmTimeEnd write SetAlarmTimeEnd;
    property ShowObjectBegin: TControl read FShowObjectBegin write FShowObjectBegin;
    property ShowObjectEnd: TControl read FShowObjectEnd write FShowObjectEnd;
    property ActiveBegin: Boolean read FActiveBegin write SetActiveBegin default false;
    property ActiveEnd: Boolean read FActiveEnd write SetActiveEnd default false;
    property OnAlarmBegin: TNotifyEvent read FOnAlarmBegin write FOnAlarmBegin;
    property OnAlarmEnd: TNotifyEvent read FOnAlarmEnd write FOnAlarmEnd;
    property MiliSeconds: Boolean read FMSec write SetMSec default false;
  end;

procedure Register;

implementation

constructor TAlarmFromTo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FTimer := TTimer.Create(Self);
  FTimer.Enabled := False;
  FFormat := '%0.2d:%0.2d:%0.2d';
//  if FAlarmTimeBegin = '' then FAlarmTimeBegin := TimeToStr(Time);
//  if FAlarmTimeEnd = '' then FAlarmTimeEnd := TimeToStr(Time);
  FInterval := 1000;
  FTimer.OnTimer := RunAlarm;
  ActiveBegin := False;
  ActiveEnd := False;
end;

destructor TAlarmFromTo.Destroy;
begin
  FTimer.Free;
  inherited Destroy;
end;

procedure TAlarmFromTo.SetInterval(Inter: Cardinal);
begin
  FInterval := Inter;
  FTimer.Interval := FInterval;
end;

procedure TAlarmFromTo.SetMSec(State: Boolean);
begin
  if State then FFormat := '%0.2d:%0.2d:%0.2d:%0.3d'
  else FFormat := '%0.2d:%0.2d:%0.2d';
  FMSec := State;
end;

procedure TAlarmFromTo.SetActiveBegin(State: Boolean);
var
  falmTime,
  falmToGo,
  ftmpTime: TDateTime;
  Kind: Boolean;
  ss: string;
begin
  if FActiveBegin <> State then FActiveBegin := State;

  if FActiveBegin then  begin
    falmTime := StrToTime(FAlarmTimeBegin);

    if Time < falmTime then
    begin
      falmToGo :=  falmTime - Time;
      Kind := True;
    end else
    begin
      ftmpTime := StrToTime('23:59:59:999');
      falmToGo := ftmpTime - Time + falmTime;
      Kind := False;
    end;

    DecodeTime(falmToGo, FHour, FMin, FSec, FMlsec);
    if not Kind then inc(FSec);

    FTimer.Enabled := True;
  end else
    FTimer.Enabled := False or ActiveEnd;
    if FTimer.Enabled then FTimer.Interval := FInterval;
end;


procedure TAlarmFromTo.SetActiveEnd(State: Boolean);
var
  falmTime,
  falmToGo,
  ftmpTime: TDateTime;
  Kind: Boolean;
  ss: string;
begin
  if FActiveEnd <> State then FActiveEnd := State;
  if FActiveEnd then
  begin
    falmTime := StrToTime(FAlarmTimeEnd);

    if Time < falmTime then
    begin
      falmToGo :=  falmTime - Time;
      Kind := True;
    end else
    begin
      ftmpTime := StrToTime('23:59:59');
      falmToGo := ftmpTime - Time + falmTime;
      Kind := False;
    end;

    DecodeTime(falmToGo, FHour, FMin, FSec, FMlsec);
    if not Kind then inc(FSec);

    FTimer.Enabled := True;
  end else
    FTimer.Enabled := False or ActiveBegin;
    if FTimer.Enabled then FTimer.Interval := FInterval;
end;

procedure TAlarmFromTo.SetAlarmTime(Tm: string);
begin
  if Length(Tm) = 2 then Tm := Tm + ':00:00' else
  if Length(Tm) = 5 then Tm := Tm + ':00';
  FAlarmTimeBegin := FormatDateTime('hh:mm:ss', StrToTime(Tm));
end;

procedure TAlarmFromTo.SetAlarmTimeEnd(Tm: string);
begin
  if Length(Tm) = 2 then Tm := Tm + ':00:00' else
  if Length(Tm) = 5 then Tm := Tm + ':00';
  FAlarmTimeEnd := FormatDateTime('hh:mm:ss', StrToTime(Tm));
end;

procedure TAlarmFromTo.RunAlarm(Sender: TObject);
begin
if FActiveBegin then AlarmBegin;
if FActiveEnd then AlarmEnd;
end;

procedure TAlarmFromTo.AlarmBegin;
var
  strHour, strMin, strSec, strInfo: string;
  MSec: Word;
begin
  if StrToTime(FAlarmTimeBegin) >= Time then
     DecodeTime(StrToTime(FAlarmTimeBegin)-Time,FHour, FMin, FSec, MSec)
  else
     DecodeTime(EncodeTime(23,59,59,999)+StrToTime(FAlarmTimeBegin)-Time,FHour, FMin, FSec, MSec);

  strHour := IntToStr(FHour);
  if FHour < 10 then strHour := '0' + strHour;
  strMin := IntToStr(FMin);
  if FMin < 10 then strMin := '0' + strMin;
  strSec := IntToStr(FSec);
  if FSec < 10 then strSec := '0' + strSec;

//  strInfo := strHour + ':' + strMin + ':' + strSec;
//  strInfo := Format('%0.2d:%0.2d:%0.2d:%0.3d',[FHour,FMin,FSec,MSec]);
  strInfo := Format(FFormat,[FHour,FMin,FSec,MSec]);

  if Assigned(FShowObjectBegin) then
  begin
    if (FShowObjectBegin is TLabel) then (FShowObjectBegin as TLabel).Caption := strInfo
    else
    if (FShowObjectBegin is TPanel) then (FShowObjectBegin as TPanel).Caption := strInfo
    else
    if (FShowObjectBegin is TButton) then (FShowObjectBegin as TButton).Caption := strInfo
    else
    if (FShowObjectBegin is TEdit) then (FShowObjectBegin as TEdit).Text := strInfo
    else  begin
            raise EAlarmException.Create('Unlisted ShowObjectBegin');
            Exit;
    end;
  end;
  if (FHour = 0) and (FMin = 0) and (FSec = 0) then
  begin
    if Assigned(FOnAlarmBegin) then FOnAlarmBegin(Self);
    ActiveBegin := False;
  end;
end;


procedure TAlarmFromTo.AlarmEnd;
var
  strHour, strMin, strSec, strInfo: string;
  MSec: Word;
begin
  if StrToTime(FAlarmTimeEnd) >= Time then
     DecodeTime(StrToTime(FAlarmTimeEnd)-Time,FHour, FMin, FSec, MSec)
  else
     DecodeTime(EncodeTime(23,59,59,999)+StrToTime(FAlarmTimeEnd)-Time,FHour, FMin, FSec, MSec);

  strHour := IntToStr(FHour);
  if FHour < 10 then strHour := '0' + strHour;
  strMin := IntToStr(FMin);
  if FMin < 10 then strMin := '0' + strMin;
  strSec := IntToStr(FSec);
  if FSec < 10 then strSec := '0' + strSec;

//  strInfo := strHour + ':' + strMin + ':' + strSec;
  strInfo := Format(FFormat,[FHour,FMin,FSec,MSec]);

  if Assigned(FShowObjectEnd) then
  begin
    if (FShowObjectEnd is TLabel) then (FShowObjectEnd as TLabel).Caption := strInfo
    else
    if (FShowObjectEnd is TPanel) then (FShowObjectEnd as TPanel).Caption := strInfo
    else
    if (FShowObjectEnd is TButton) then (FShowObjectEnd as TButton).Caption := strInfo
    else
    if (FShowObjectEnd is TEdit) then (FShowObjectEnd as TEdit).Text := strInfo
    else  begin
            raise EAlarmException.Create('Unlisted ShowObjectEnd');
            Exit;
    end;
  end;
  if (FHour = 0) and (FMin = 0) and (FSec = 0) then
  begin
    if Assigned(FOnAlarmEnd) then FOnAlarmEnd(Self);
    ActiveEnd := False;
  end;
end;


procedure Register;
begin
  RegisterComponents('FormaSoft', [TAlarmFromTo]);
end;


end.